
/* Chrysalide - Outil d'analyse de fichiers binaires
 * elf-int.h - prototypes pour les structures internes du format ELF
 *
 * Copyright (C) 2008-2012 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  OpenIDA is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  OpenIDA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _FORMAT_ELF_ELF_INT_H
#define _FORMAT_ELF_ELF_INT_H


#include "elf.h"
#include "elf_def.h"
#include "../executable-int.h"
#include "../../common/endianness.h"



/* Format d'exécutable générique (instance) */
struct _GElfFormat
{
    GExeFormat parent;                      /* A laisser en premier        */

    elf_header header;                      /* En-tête du format           */
    bool is_32b;                            /* Format du binaire           */
    SourceEndian endian;                    /* Boutisme du format          */

};

/* Format d'exécutable générique (classe) */
struct _GElfFormatClass
{
    GExeFormatClass parent;                 /* A laisser en premier        */

};



/* Procède à la lecture de l'en-tête d'un contenu binaire ELF. */
bool read_elf_header(GElfFormat *, elf_header *, bool *, SourceEndian *);

/* Procède à la lecture d'une en-tête de programme ELF. */
bool read_elf_program_header(const GElfFormat *, phys_t, elf_phdr *);

/* Procède à la lecture d'une en-tête de section ELF. */
bool read_elf_section_header(const GElfFormat *, phys_t, elf_shdr *);

/* Procède à la lecture d'une entrée de type 'DYNAMIC' ELF. */
bool read_elf_dynamic_entry(const GElfFormat *, phys_t, elf_dyn *);

/* Procède à la lecture d'un symbole ELF. */
bool read_elf_symbol(const GElfFormat *, phys_t *, elf_sym *);

/* Procède à la lecture d'une relocalisation ELF. */
bool read_elf_relocation(const GElfFormat *, phys_t *, elf_rel *);



#endif  /* _FORMAT_ELF_ELF_INT_H */
